from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.backends.openssl import backend as openssl
import os
import base64

class KeySalt:
  def __init__(self, key, salt):
    self.key = key
    self.salt = salt

def pw2key(pw, salt=None):
  if salt == None:
    salt = os.urandom(16)
  kdf = PBKDF2HMAC(algorithm=hashes.SHA256(), length=32, salt=salt, iterations=100000, backend=openssl)
  key = base64.urlsafe_b64encode(kdf.derive(pw.encode()))
  return KeySalt(key, salt)
    
def encrypt(key, plaintext):
  f = Fernet(key)
  cipher = f.encrypt(plaintext.encode())
  return cipher
  
def decrypt(key, cipher):
  f = Fernet(key)
  plaintext = f.decrypt(cipher).decode()
  return plaintext
