#!/usr/bin/env python3
# coding: utf-8

import json
import config
import sys
import gnupg
from jj_gpg import decrypt, recoverPw, sign_file
import re

CAND_NUM = config.CAND_NUM
DESC = config.DESC


def process_vote(rows):
    values = {}
    numyes = 0
    ok = True
    for (cs, vs) in rows:
        c = int(cs)
        v = int(vs)
        if v not in election_values:
            ok = False
            print("   Hodnota mimo rozsah")
        elif c not in cand_ids:
            ok = False
            print("   Neznámy kandidát")
        elif c in values:
            ok = False
            print("   Duplicitný kandidát")
        else:
            values[c] = v
            if v == 1:
                numyes += 1
    if numyes == 0:
        print("   Prázdny lístok")
        ok = False
    elif numyes > CAND_NUM:
        ok = False
        print("   Priveľa zvolených kandidátov")
    
    if ok:
        return values
    else:
        return None        


# We will recover the gpg passphrase
recoverPw()

try:
    with open('data/votes.txt', 'r') as file:
        votes = json.load(file)  # creates a list of votes out of the json file
except FileNotFoundError:
    sys.exit("Požadovaný súbor votes.txt sa nenašiel")

try:
    with open('data/candidates.txt', 'r') as file:
        candidates = json.load(file)
except FileNotFoundError:
    sys.exit("Požadovaný súbor candidates.txt sa nenašiel")

election_values = [1, 3]
cand_ids = []
for c in candidates:
    cand_ids.append(c[0])

platnych = 0
neplatnych = 0

election_yes = {}
election_no = {}
election_abst = {}

for c in cand_ids:
    election_yes[c] = 0
    election_no[c] = 0
    election_abst[c] = 0

plainvotes = []

corrupted = []

for v in votes:
    if v[0] == "0":
        continue
    plain_v = decrypt(v[0])
    if plain_v is None:
        print("Nedešifrovateľný hlas")
        neplatnych += 1
        plainvotes.append("Nedešifrovateľný lístok")
        corrupted.append(v[0])
        continue
    plainvotes.append(plain_v)
    if not re.match('^(<[0-9]*#[0-9]*>)+$', plain_v):
        print("Syntakticky chybný neplatný hlas")
        neplatnych +=1
        continue
    # Sem sa dostaneme len v pripade, ze mame hlasovaci listok v syntakticky spravnom tvare
    rows = re.findall('<([0-9]*)#([0-9]*)>', plain_v)
    values = process_vote(rows)
    if values is None:
        print("Neplatný lístok")
        neplatnych += 1
    else:
        platnych += 1
        for c,h in values.items():
            if h == 1:
                election_yes[c] += 1
            elif h == 2:
                election_no[c] += 1
            elif h == 3:
                election_abst[c] += 1
            else:
                print("Nedefinovaný typ hlasu")
                exit()

plainvotes.sort()

total_yes = 0
total_no = 0
total_abst = 0
for c in cand_ids:
    total_yes += election_yes[c]
    total_no += election_no[c]
    total_abst += election_abst[c]

print("Celkom za: %d, celkom proti: %d, celkom zdržaných: %d, spolu: %d, má byť: %d" % (total_yes, total_no, total_abst, total_yes+total_no+total_abst, platnych*len(candidates)))
print("Platných lístkov: %d, neplatných lístov: %d, spolu: %d, má byť: %d" % (platnych, neplatnych, platnych+neplatnych, len(votes)))
if (total_yes+total_no+total_abst == platnych*len(candidates)) and (platnych+neplatnych == len(votes)):
    print("Počty sedia")
else:
    print("Počty NESEDIA !!!!!!!!!!!!")
    exit()

levels = [ None, 'bakalársky', 'magisterský', 'doktorandský' ]
# print the final results
with open("data/results.txt", 'w') as res_file:
    res_file.write("Výsledky volieb %s:\n" % DESC)
    res_file.write("číslo|meno|program|stupeň|rok|počet hlasov\n")
    for c in candidates:
        res_file.write(str(c[0]) + "|" + c[1] + "|" + c[2] + "|" + levels[c[3]] + "|" + str(c[4]) + "|" + str(election_yes[c[0]]) + "\n")
    
with open("data/results.txt", 'r') as res_file:
    print(res_file.read())
    
with open("data/results.txt", "rb") as res_file:
    signed = sign_file(res_file)
    if signed:
        with open("data/results.txt.sig", "w") as sig_file:
            sig_file.write(signed)

with open("data/plainvotes.txt", "w") as f:
    for v in plainvotes:
        f.write(v);
        f.write('\n');

with open("data/votes.txt", "rb") as f:
    signed = sign_file(f)
    if signed:
        with open("data/votes.txt.sig", "w") as sig_file:
            sig_file.write(signed)

with open("data/plainvotes.txt", "rb") as f:
    signed = sign_file(f)
    if signed:
        with open("data/plainvotes.txt.sig", "w") as sig_file:
            sig_file.write(signed)

with open("data/voters.txt", "rb") as f:
    signed = sign_file(f)
    if signed:
        with open("data/voters.txt.sig", "w") as sig_file:
            sig_file.write(signed)

with open("data/corrupted.txt", "w") as f:
    for v in corrupted:
        f.write(v)
        f.write("\n")
